/*

    ThermostatQA - test framework for Thermostat Monitoring Tool

    Copyright 2013 Red Hat, Inc.

This file is part of ThermostatQA

ThermostatQA is distributed under the GNU General Public License,
version 2 or any later version (with a special exception described
below, commonly known as the "Classpath Exception").

A copy of GNU General Public License (GPL) is included in this
distribution, in the file COPYING.

Linking ThermostatQA code with other modules is making a combined work
based on ThermostatQA.  Thus, the terms and conditions of the GPL
cover the whole combination.

As a special exception, the copyright holders of ThermostatQA give you
permission to link this code with independent modules to produce an
executable, regardless of the license terms of these independent
modules, and to copy and distribute the resulting executable under
terms of your choice, provided that you also meet, for each linked
independent module, the terms and conditions of the license of that
module.  An independent module is a module which is not derived from
or based on ThermostatQA code.  If you modify ThermostatQA, you may
extend this exception to your version of the software, but you are
not obligated to do so.  If you do not wish to do so, delete this
exception statement from your version.
 */

package org.thermostat.qa.reporter;



import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;



/**
 * Generator for the "index.html" page which contains the daily report.
 *
 * @author Pavel Tisnovsky <ptisnovs@redhat.com>
 */
public class IndexPageGenerator implements CommonGenerator
{

    /**
     * Generate the "index.html" page using template and current test results.
     * 
     * @param testResults
     *            map containing all the test results.
     * 
     * @param params
     *            parameters passed to the generator.
     */
    public static void generate(Map<String, Map<String, List<String>>> testResults, CommandLineParameters params)
    {
        String date = params.getDate();
        // try to find results for the given date
        if (!testResults.containsKey(params.getDate()))
        {
            throw new RuntimeException("Results for " + date + " don't exist");
        }
        // get the results for the given date
        Map<String, List<String>> testResultForDate = testResults.get(date);
        TestResult testResult = TestResult.readSummary(testResultForDate);
        testResult.setDate(date);
        // and create "index.html" page
        createIndexPageFromTemplate(testResult, params);
    }

    /**
     * Create "index.html" page from the template using test results.
     * 
     * @param testResult
     *            containing the test result.
     * @param params
     *            parameters passed to the generator.
     */
    private static void createIndexPageFromTemplate(TestResult testResult, CommandLineParameters params)
    {
        // read the template
        List<String> template = readTemplate(params);
        List<String> out = new LinkedList<String>();

        // iterate through whole template
        // and replace all placeholders by its values
        Pattern pattern = Pattern.compile("\\$\\{.*\\}");
        for (String templateLine : template)
        {
            out.add(replacePlaceholder(testResult, pattern, templateLine));
        }
        // write list of string to a file with given name
        FileUtils.writeTextFile(params.getReportDir() + "/index.html", out);
    }

    /**
     * Read template from the template directory.
     *
     * @param params
     *            parameters passed to the generator.
     * @return template stored as a list of Strings
     */
    private static List<String> readTemplate(CommandLineParameters params) {
        return FileUtils.readTextFile(params.getTemplateDir() + "/index.html");
    }

    /**
     * Replace placeholder ${PLACEHOLDER_NAME} by its value which is read using
     * TestResult.getPlaceholderName().
     * 
     * @param testResult
     *            containing the test result.
     * @param pattern
     *            pattern for placeholder in a template
     * @param templateLine
     *            line read from template, where this method search for any
     *            placeholder
     * @return placeholder value
     */
    private static String replacePlaceholder(TestResult testResult, Pattern pattern, String templateLine)
    {
        Matcher matcher = pattern.matcher(templateLine);
        // if some pattern ${PLACEHOLDER} is found on the template line,
        // it is replaced by the result value of method
        // TestResult.getPlaceholder()
        if (matcher.find())
        {
            String matcherGroup = matcher.group();
            // convert placeholder name into getter name
            String placeholderMethodName = StringUtils.getPlaceholderMethodName(matcherGroup);
            try
            {
                // invoke given getter
                Method method = testResult.getClass().getMethod(placeholderMethodName);
                Object result = method.invoke(testResult);
                // replace placeholder by the value returned by getter
                if (result != null)
                {
                    return templateLine.replace(matcherGroup, result.toString());
                }
            }
            catch (SecurityException e)
            {
                e.printStackTrace();
            }
            catch (NoSuchMethodException e)
            {
                e.printStackTrace();
            }
            catch (IllegalArgumentException e)
            {
                e.printStackTrace();
            }
            catch (IllegalAccessException e)
            {
                e.printStackTrace();
            }
            catch (InvocationTargetException e)
            {
                e.printStackTrace();
            }
        }
        return templateLine;
    }
}
